local fft = require("fft")
local ffi = require("ffi")
local complex = require("complex")

local audio = love.sound.newSoundData("music.mp3")

local sampleRate = audio:getSampleRate()
local numSamples = audio:getSampleCount()
local RESOLUTION = 2048

local dest = love.audio.newQueueableSource(sampleRate, audio:getBitDepth(), audio:getChannelCount())

-- used for the display
local spectrum_left = ffi.new("double["..(RESOLUTION/2).."]")
local spectrum_right = ffi.new("double["..(RESOLUTION/2).."]")

local WINDOW_WIDTH = love.graphics.getWidth()
local WINDOW_HEIGHT = love.graphics.getHeight()

-- buffer used for queueing
local curSampleData = love.sound.newSoundData(RESOLUTION, sampleRate, audio:getBitDepth(), audio:getChannelCount())

-- position into audio
local curSample = 0

-- using ffi arrays don't work, however using Lua tables does
--local data_l = ffi.new("lua_complex["..(RESOLUTION).."]")
--local data_r = ffi.new("lua_complex["..(RESOLUTION).."]")
local data_l = {}
local data_r = {}

local function updateBuf()
    local numChannels = audio:getChannelCount()
    
    -- get slice of audio samples of size RESOLUTION
    for i=0, RESOLUTION-1 do
        local sample_l = audio:getSample(curSample, 1)
        local sample_r = audio:getSample(curSample, 2)

        data_l[i] = complex.new(sample_l)
        data_r[i] = complex.new(sample_r)

        curSample = curSample + 1
    end

    -- get frequency data of samples
    fft.inplace(data_l, RESOLUTION)
    fft.inplace(data_r, RESOLUTION)

    -- frequency data is located in the imaginary components of the first half of the array
    local halfres = RESOLUTION/2

    for i=0, halfres-1 do
        -- display frequency data
        spectrum_left[i] = data_l[i].imag / RESOLUTION
        spectrum_right[i] = data_r[i].imag / RESOLUTION


        -- apply a filter
        -- this creates a clicking sound for some reason?
        -- need to find out how to fix that
        local factor = i / halfres
    
        data_l[i] = data_l[i] * factor
        data_r[i] = data_r[i] * factor
        data_l[RESOLUTION - i - 1] = data_l[RESOLUTION - i - 1] * factor
        data_r[RESOLUTION - i - 1] = data_r[RESOLUTION - i - 1] * factor
    end

    -- rebuild samples from modified frequency data
    fft.inplace(data_l, RESOLUTION, true)
    fft.inplace(data_r, RESOLUTION, true)
    
    -- set sample data
    for i=0, RESOLUTION-1 do
        curSampleData:setSample(i, 1, data_l[i].real)
        curSampleData:setSample(i, 2, data_r[i].real)
    end
end

function love.update(dt)
    if dest:getFreeBufferCount() > 0 then
        updateBuf()
        dest:queue(curSampleData)
        dest:play()
    end
end

function love.draw(d)
    -- draw spectrum
    local points = {}
    local pointsi = 1

    local halfres = RESOLUTION/2
    for i=0, halfres-1 do
        points[pointsi] = math.log10( i+1 ) * 200
        --points[pointsi] = (i / halfres) * WINDOW_WIDTH
        points[pointsi+1] = spectrum_left[i] * 8 * WINDOW_HEIGHT/2 + WINDOW_HEIGHT/2
        pointsi=pointsi+2
    end

    love.graphics.setColor(1, 1, 1)
    love.graphics.setLineWidth(1)
    love.graphics.line(points)
end

